@@ -15,7 +15,7 @@
 ##  along with this program; if not, write to the Free Software
 ##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-##  $Id: testLogger.r,v 1.6 2004/05/19 10:48:39 kjuen Exp $
+##  $Id: testLogger.r,v 1.9 2004/09/09 10:21:22 kjuen Exp $
 
 
 .newTestLogger <- function(useOwnErrorHandler) {
@@ -27,23 +27,26 @@
 
   ## private data:
   ## -----------------------
-  testData <- list()
-  class(testData) <- "RUnitTestData"
-  currentTestSuiteName <- NULL
-  currentSourceFileName <- NULL
-  currentTraceBack <- NULL
+  .testData <- list()
+  class(.testData) <- "RUnitTestData"
+  .currentTestSuiteName <- NULL
+  .currentSourceFileName <- NULL
 
-  isFailure <- FALSE
-  checkNo <- 0
+  ## book keeping variables for individual test functions
+  ## can be reset by cleanup function
+  .currentTraceBack <- NULL
+  .failure <- FALSE
+  .deactivationMsg <- NULL   ## if non-NULL test function is deactivated
+  .checkNum <- 0
 
   ## define own error handler
   ## -----------------------
   errorHandler<-function() {
     dump.frames();
     frameLimit<-length(last.dump)-1
-    currentTraceBack <<- NULL
+    .currentTraceBack <<- NULL
     for(i in 1:frameLimit)
-      currentTraceBack <<- c(currentTraceBack, names(last.dump)[i])
+      .currentTraceBack <<- c(.currentTraceBack, names(last.dump)[i])
   }
   if(useOwnErrorHandler) {
     options(error=errorHandler)
@@ -57,7 +60,7 @@
     ##@bdescr
     ## returns the protocol data collected during the test runs
     ##@edescr
-    return(testData)
+    return(.testData)
   }
 
   setCurrentTestSuite <- function(testSuite) {
@@ -67,14 +70,17 @@
     ##@in testSuite : [testSuite - list] the current testSuite
 
     if(is.null(testSuite)) {
-      currentTestSuiteName <<- NULL
+      .currentTestSuiteName <<- NULL
     }
     else {
-      if(is.element(testSuite$name, names(testData))) {
+      if(is.element(testSuite$name, names(.testData))) {
         stop(paste("Duplicate test suite:", testSuite$name))
       }
-      currentTestSuiteName <<- testSuite$name
-      testData[[testSuite$name]] <<- list(nTestFunc=0, nErr=0, nFail=0,
+      .currentTestSuiteName <<- testSuite$name
+      .testData[[testSuite$name]] <<- list(nTestFunc=0,
+                                          nDeactivated=0,
+                                          nErr=0,
+                                          nFail=0,
                                           dirs=testSuite$dirs,
                                           testFileRegexp=testSuite$testFileRegexp,
                                           testFuncRegexp=testSuite$testFuncRegexp,
@@ -89,11 +95,11 @@
     ##@in sourceFileName : [character] name of current source file
 
     if(is.null(sourceFileName)) {
-      currentSourceFileName <<- NULL
+      .currentSourceFileName <<- NULL
     }
     else {
-      currentSourceFileName <<- sourceFileName
-      testData[[currentTestSuiteName]]$sourceFileResults[[sourceFileName]] <<- list()
+      .currentSourceFileName <<- sourceFileName
+      .testData[[.currentTestSuiteName]]$sourceFileResults[[sourceFileName]] <<- list()
     }
   }
 
@@ -104,9 +110,9 @@
     ##@in testFuncName : [character] name of test function
     ##@in secs : [numeric] time in seconds needed by the test function to complete
 
-    testData[[currentTestSuiteName]]$nTestFunc <- 1 + testData[[currentTestSuiteName]]$nTestFunc
+    .testData[[.currentTestSuiteName]]$nTestFunc <<- 1 + .testData[[.currentTestSuiteName]]$nTestFunc
 
-    testData[[currentTestSuiteName]]$sourceFileResults[[currentSourceFileName]][[testFuncName]] <<-
+    .testData[[.currentTestSuiteName]]$sourceFileResults[[.currentSourceFileName]][[testFuncName]] <<-
       list(kind="success", time=secs)
   }
 
@@ -117,28 +123,70 @@
     ##@in testFuncName : [character] name of test function
     ##@in errorMsg : [character] the error message
 
-    testData[[currentTestSuiteName]]$nTestFunc <- 1 + testData[[currentTestSuiteName]]$nTestFunc
-    testData[[currentTestSuiteName]]$nErr <- 1 + testData[[currentTestSuiteName]]$nErr
+    .testData[[.currentTestSuiteName]]$nTestFunc <<- 1 + .testData[[.currentTestSuiteName]]$nTestFunc
+    .testData[[.currentTestSuiteName]]$nErr <<- 1 + .testData[[.currentTestSuiteName]]$nErr
 
-    testData[[currentTestSuiteName]]$sourceFileResults[[currentSourceFileName]][[testFuncName]] <<-
-      list(kind="error", msg=errorMsg, traceBack=currentTraceBack)
-    currentTraceBack <<- NULL
+    .testData[[.currentTestSuiteName]]$sourceFileResults[[.currentSourceFileName]][[testFuncName]] <<-
+      list(kind="error", msg=errorMsg, traceBack=.currentTraceBack)
   }
 
-  addFailure <- function(testFuncName, failureMsg, checkNum) {
+  addFailure <- function(testFuncName, failureMsg) {
     ##@bdescr
     ## add a test function that generated an error.
     ##@edescr
     ##@in testFuncName : [character] name of test function
     ##@in failureMsg : [character] the failure message
-    ##@in checkNum: [integer] number of check function that failed.
 
-    testData[[currentTestSuiteName]]$nTestFunc <- 1 + testData[[currentTestSuiteName]]$nTestFunc
-    testData[[currentTestSuiteName]]$nFail <- 1 + testData[[currentTestSuiteName]]$nFail
+    .testData[[.currentTestSuiteName]]$nTestFunc <<- 1 + .testData[[.currentTestSuiteName]]$nTestFunc
+    .testData[[.currentTestSuiteName]]$nFail <<- 1 + .testData[[.currentTestSuiteName]]$nFail
 
-    testData[[currentTestSuiteName]]$sourceFileResults[[currentSourceFileName]][[testFuncName]] <<-
-      list(kind="failure", msg=failureMsg, checkNo=checkNum, traceBack=NULL)  ## traceBack is useless in this case
-    currentTraceBack <<- NULL
+    .testData[[.currentTestSuiteName]]$sourceFileResults[[.currentSourceFileName]][[testFuncName]] <<-
+      list(kind="failure", msg=failureMsg, checkNum=.checkNum, traceBack=NULL)  ## traceBack is useless in this case
+  }
+
+  addDeactivated <- function(testFuncName) {
+    ##@bdescr
+    ## add a deactivated test function that generated an error.
+    ##@edescr
+    ##@in testFuncName : [character] name of test function
+
+
+    .testData[[.currentTestSuiteName]]$nDeactivated <<- 1 + .testData[[.currentTestSuiteName]]$nDeactivated
+    .testData[[.currentTestSuiteName]]$sourceFileResults[[.currentSourceFileName]][[testFuncName]] <<-
+      list(kind="deactivated", msg=.deactivationMsg)
+  }
+
+
+  cleanup <- function() {
+    ##@bdescr
+    ## reset book keeping variables like .failure, ...
+    ## should be called before each test function execution
+    ##@edescr
+
+    .currentTraceBack <<- NULL
+    .failure <<- FALSE
+    .deactivationMsg <<- NULL
+    .checkNum <<- 0
+  }
+
+  isFailure <- function() {
+    return(.failure)
+  }
+
+  setFailure <- function() {
+    .failure <<- TRUE
+  }
+
+  isDeactivated <- function() {
+    return(!is.null(.deactivationMsg))
+  }
+
+  setDeactivated <- function(msg) {
+    .deactivationMsg <<- msg
+  }
+
+  incrementCheckNum <- function() {
+    .checkNum <<- 1 + .checkNum
   }
 
   return(list(getTestData=getTestData,
@@ -146,26 +194,39 @@
               setCurrentSourceFile=setCurrentSourceFile,
               addSuccess=function(testFuncName, secs) addSuccess(testFuncName, secs),
               addError=function(testFuncName, errorMsg) addError(testFuncName, errorMsg),
-              addFailure=function(testFuncName, failureMsg, checkNum) addFailure(testFuncName, failureMsg, checkNum),
+              addFailure=function(testFuncName, failureMsg) addFailure(testFuncName, failureMsg),
+              addDeactivated=function(testFuncName) addDeactivated(testFuncName),
               isFailure=isFailure,
-              checkNo=checkNo))
+              setFailure=setFailure,
+              isDeactivated=isDeactivated,
+              setDeactivated=function(msg) setDeactivated(msg),
+              incrementCheckNum=incrementCheckNum,
+              cleanup=cleanup))
 }
 
 
 
 
 
-## tools to handle the testData listlistlist
-.getErrors <- function(testData) {
+
+
+getErrors <- function(testData) {
+  ##@bdescr
+  ##  tools to handle the testData listlistlist
+  ##
+  ##@edescr
+  ##
+  ##@in testData : [list] S3 RUnitTestData class object
+
   if(class(testData) != "RUnitTestData") {
-    stop(".getErrors needs an object of class 'RUnitTestData' as argument.")
+    stop("getErrors needs an object of class 'RUnitTestData' as argument.")
   }
-  ret <- list(nErr=0, nFail=0, nTestFunc=0)
+  ret <- list(nErr=0, nDeactivated=0, nFail=0, nTestFunc=0)
   for(i in seq(length=length(testData))) {
     ret$nErr <- ret$nErr + testData[[i]]$nErr
+    ret$nDeactivated <- ret$nDeactivated + testData[[i]]$nDeactivated
     ret$nFail <- ret$nFail + testData[[i]]$nFail
     ret$nTestFunc <- ret$nTestFunc + testData[[i]]$nTestFunc
   }
   return(ret)
 }
-
